<?php

namespace InSegment\WorkersFramework\Providers;

use Shavshukov\EventAppBase\Service\Provider;
use Shavshukov\EventAppBase\Service\Container;
use React\Promise;

use InSegment\WorkersFramework\Interfaces\QueueConnectorInterface;
use InSegment\WorkersFramework\Queues\ConnectionsConfig;
use InSegment\WorkersFramework\Queues\ConnectionsList;

class QueueConnectionServiceProvider extends Provider
{
    
    /**
     * {@inheritDoc}
     */
    public function register(Container $container, ...$registerArguments)
    {
        $container->bind('queue-connections-list', function () {
            return $this->invokeConnectors($this->app->config()->getConnectionsConfig());
        }, true);
    }
    
    /**
     * Invoke connectors to establish connections
     * 
     * @param \InSegment\WorkersFramework\Queues\ConnectionsConfig $config
     * @return \React\Promise\PromiseInterface<\InSegment\WorkersFramework\Queues\ConnectionsList>
     */
    protected function invokeConnectors(ConnectionsConfig $config): Promise\PromiseInterface
    {
        $loop = $this->app->getLoop();
        return Promise\all(array_map(function (QueueConnectorInterface $connector) use ($loop) {
            return $connector->createDriver($loop);
        }, $config->getConnectors()))->then(function (array $connections) use ($config) {
            return new ConnectionsList($config, $connections);
        });
    }
    
}
