<?php

namespace InSegment\WorkersFramework\Services;

use Shavshukov\EventAppBase\ApplicationInterface;
use React\Promise;

use InSegment\WorkersFramework\Interfaces\ConnectionListInterface;
use InSegment\WorkersFramework\Queues\Worker;

class Declarator
{
    /**
     * Application instance
     *
     * @var \Shavshukov\EventAppBase\ApplicationInterface
     */
    protected $app;
    
    /**
     * Instance of queue driver connected to the channel
     *
     * @var \InSegment\WorkersFramework\Interfaces\ConnectionListInterface
     */
    protected $connections;
    
    /**
     * Constructor
     * 
     * @param \Shavshukov\EventAppBase\ApplicationInterface $app
     * @param \InSegment\WorkersFramework\Interfaces\ConnectionListInterface $connections
     * @throws \Exception
     */
    public function __construct(ApplicationInterface $app, ConnectionListInterface $connections)
    {
        $this->app = $app;
        $this->connections = $connections;
    }
    
    /**
     * Declare necessary queues
     * 
     * @return \React\Promise\PromiseInterface
     */
    public function declare(): Promise\PromiseInterface
    {
        return $this->declareIncomingQueue()->then(function () {
            $promises = [];
            $nodeName = $this->app->config('node_name');
            foreach ($this->app->config('workers', []) as $workerName => $workerConfig) {
                $promises[] = $this->declareQueueForWorker(new Worker($workerName, $workerConfig, $nodeName));
            }
            
            return Promise\all($promises);
        })->then(function () {
            echo "Done.";
        });
    }
    
    /**
     * Declare main input queue
     * 
     * @return \React\Promise\PromiseInterface
     */
    protected function declareIncomingQueue(): Promise\PromiseInterface
    {
        echo "Declaring main incoming and service queues...\n";
        
        return Promise\all(array_map(function ($queueConfigName) {
            $queueName = $this->app->config($queueConfigName);
            return $this->connections->declareQueue($queueName)->doDeclare();
        }, ['main_queue', 'service_queue']));
    }
    
    /**
     * Declare queue for worker
     * 
     * @param \InSegment\WorkersFramework\Queues\Worker $worker
     * @return \React\Promise\PromiseInterface
     */
    protected function declareQueueForWorker(Worker $worker): Promise\PromiseInterface
    {
        echo "Declaring queue: {$worker->getQueueName()} for worker {$worker->getName()}...\n";
        
        return $worker->getDeclareStructure($this->connections)->declareAll();
    }
}
